module XMLGram where
import ParseLib
import XMLLex

data Node = Node	   	Tag [Node] Tag
          | Content 	String
          | ClosedNode  Tag
          deriving (Show, Eq)

data Tag = OpenTag		String [Attribute]
		 | SluitTag		String
         | ClosedTag    String [Attribute]
		 deriving (Show, Eq)

data Attribute = Attribute String String String
          deriving (Show, Eq)
          
parenthesized p = pack (symbol PHOpen) p (symbol PHSluit)

pNode :: Parser Token Node
pNode =  matchingTag
         <$> pOpenTag
         <*> many pNode
		 <*> pSluitTag
     <|> (\(Name c) -> Content c)
         <$> sName
     <|> (\t -> ClosedNode t)
         <$> pClosedTag
  where matchingTag o@(OpenTag n1 _) c s@(SluitTag n2) | n1 == n2 = Node o c s
                                                       | otherwise = Node o c s
		 
pOpenTag :: Parser Token Tag
pOpenTag = (\((Name n),xs) -> OpenTag n xs)
           <$> parenthesized (totuple <$> sName <*> many pAttribute)

pSluitTag :: Parser Token Tag
pSluitTag = (\(Name n) -> SluitTag n)
            <$> pack (pi22 <$> symbol PHOpen <*> symbol ForwardSlash) sName (symbol PHSluit)

pClosedTag :: Parser Token Tag
pClosedTag = (\((Name n),a) -> ClosedTag n a)
             <$> pack (symbol PHOpen) (totuple <$> sName <*> many pAttribute) (pi22 <$> symbol ForwardSlash <*> symbol PHSluit)
             

pAttribute :: Parser Token Attribute
pAttribute = (\(Name n) _ (Quote q) (Name v) (Quote q2) -> Attribute n v q)
             <$> sName
             <*> symbol Equals
             <*> sQuote
             <*> sName
             <*> sQuote

totuple :: a -> b -> (a,b)
totuple a b = (a,b)