module XMLLex where
import Char
import ParseLib

data Token = PHOpen   | PHSluit       -- Punthaken       <>
           | Quote String             -- Enkele en dubbele quote   ' "
           | ForwardSlash
		   | Equals
		   | Name String			  -- any string consisting of only alphabetic characters
		   | Value String			  -- any string consisting of alphanumeric characters
           deriving (Eq, Show)

terminals =
    [( PHOpen       , "<"      )
    ,( PHSluit      , ">"      )
    ,( ForwardSlash , "/"  )
    ,( Equals       , "="  )
    ]

lexWhiteSpace :: Parser Char String
lexWhiteSpace = greedy (satisfy isSpace)

lexTerminal :: Parser Char Token
lexTerminal = choice (map f terminals)
    where f (t,s) = const t <$> token s

lexToken :: Parser Char Token
lexToken = choice [ lexTerminal
                  , lexName
                  , lexQuote
                  ]

lexicalScanner :: Parser Char [Token]
lexicalScanner = (\_ x->x) <$> lexWhiteSpace <*> many ((\x _->x) <$> lexToken <*> lexWhiteSpace)

-- TODO: remove first part and go straight to greedy
lexName :: Parser Char Token
lexName = (\x xs -> Name (x:xs)) <$> satisfy isAlphaNum <*> greedy (satisfy isAlphaNum)

lexQuote :: Parser Char Token
lexQuote = (\x -> Quote x)
           <$> (token "\'" <|> token "\"")

sName :: Parser Token Token
sName = satisfy isName
       where isName (Name _) = True
             isName _        = False

sQuote :: Parser Token Token
sQuote = satisfy isQuote
       where isQuote (Quote _) = True
             isQuote _         = False